#include <iostream>
#include <algorithm>
#include <vector>
#include <array>
#include <random>
#include <cmath>
#include <utility>

#include "treeq.h"
#include "treeq.cpp"

static std::vector<std::vector<int>> dist;
static std::vector<std::pair<int, int>> output;
static int Q = 0;

void check_vertex(int x) {
    if(x < 1 || x >= dist.size()) {
        std::cerr << "Invalid vertex " << x << std::endl;
        std::cout << -3 << std::endl;
        exit(0);
    }
}

void answer(int u, int v) {
    if(output.size() == dist.size() - 2) {
        std::cerr << "Too many edges" << std::endl;
        std::cout << -2 << std::endl;
        exit(0);
    }

    check_vertex(u);
    check_vertex(v);

    if(u > v) {
        std::swap(u, v);
    }

    output.push_back({u, v});
}

static void dfs(
    int u, int p, std::vector<int> &d,
    std::vector<std::vector<int>> &adj
) {
    for(int v: adj[u]) {
        if(v != p) {
            d[v] = d[u] + 1;
            dfs(v, u, d, adj);
        }
    }
};

bool query(std::vector<int> S, int x) {
    if(S.size() < 2 || S.size() > (int)dist.size() - 1) {
        std::cerr << "Invalid query" << std::endl;
        std::cout << -4 << std::endl;
        exit(0);
    }

    check_vertex(x);
    for(int u: S) {
        check_vertex(u);
    }

	++Q;
    for(int i=0; i<S.size(); i++) {
        for(int j=i+1; j<S.size(); j++) {
            if(S[i] == S[j]) {
                std::cerr << "Invalid query" << std::endl;
                std::cout << -4 << std::endl;
                exit(0);

            }
        }
    }

    for(int u: S) {
        for(int v: S) {
            if(dist[u][v] == dist[u][x] + dist[x][v]) {
                return true;
            }
        }
    }

    return false;
}

int main() {
    int n;
    std::cin >> n;

    std::vector<std::vector<int>> adj;
    adj.assign(n + 1, std::vector<int>());

    std::vector<std::pair<int, int>> edges;

    for(int i = 2; i <= n; i++) {
        int u=0, v;
        for(int f=1;f<=1;f++){
        u=max(u,rand()%(i-1)+1);
        }
        v=i;
        adj[u].push_back(v);
        adj[v].push_back(u);

        if(u > v) {
            std::swap(u, v);
        }
        edges.push_back({u, v});
    }

    dist.assign(n + 1, std::vector<int>(n + 1, 0));
    for(int src = 1; src <= n; src++) {
        dfs(src, -1, dist[src], adj);
    }

    solve(n);

    std::sort(edges.begin(), edges.end());
    std::sort(output.begin(), output.end());

    if(edges != output) {
        std::cerr << "Wrong answer" << std::endl;
        std::cout << 0 << " " << Q << std::endl;
    } else {
        std::cerr << "Correct answer" << std::endl;
        std::cout << 1 << " " << Q << std::endl;
    }

    return 0;
}
