#include "treeq.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <random>

typedef long long llong;
const int MAXN = 100000 + 10;
const int INF  = 1e9;

int n;
struct DSU
{
    int par[MAXN];
    int dep[MAXN];

    void build()
    {
        std::iota(par + 1, par + 1 + n, 1);
        std::fill(dep + 1, dep + 1 + n, 1);
    }

    int find(int node)
    {
        if (par[node] == node) return par[node];
        return par[node] = find(par[node]);
    }

    void connect(int u, int v)
    {
        u = find(u);
        v = find(v);

        if (u == v)
        {
            return;
        }

        if (dep[u] > dep[v])
        {
            std::swap(u, v);
        }

        if (dep[u] == dep[v])
        {
            dep[v]++;
        }

        par[u] = v;
    }

};

std::mt19937 mt(123);
void reccc(std::vector <int> nodes)
{
    if (nodes.size() == 0)
    {
        return;
    }

    std::shuffle(nodes.begin(), nodes.end(), mt);
    if (nodes.size() == 1)
    {
        return;
    }

    if (nodes.size() == 2)
    {
        answer(nodes[0], nodes[1]);
        return;
    }

    DSU dsu;
    dsu.build();

    std::vector <int> roots;
    std::vector <std::pair <int,int>> toAdd;
    roots.push_back(nodes[1]);

    for (int i = 2 ; i < nodes.size() ; ++i)
    {
        roots.push_back(nodes[0]);
        if (query(roots, nodes[i]))
        {
            roots.pop_back();
            int l = -1, r = roots.size(), mid;
            while (l < r - 1)
            {
                std::vector <int> curr;
                mid = (l + r) / 2;
                curr.push_back(nodes[0]);
                for (int j = 0 ; j <= mid ; ++j)
                {
                    curr.push_back(roots[j]);
                }

                if (!query(curr, nodes[i]))
                {
                    l = mid;
                } else
                {
                    r = mid;
                }
            }

            if (r == roots.size())
            {
                roots.push_back(nodes[i]);
            } else
            {
                dsu.connect(roots[r], nodes[i]);
                roots[r] = nodes[i];
            }
        } else
        {
            roots.pop_back();
            std::vector <int> curr;
            curr.push_back(nodes[0]);
            curr.push_back(nodes[i]);

            for (int j = 0 ; j < roots.size() ; ++j)
            {
                if (query(curr, roots[j]))
                {
                    dsu.connect(nodes[i], roots[j]);
                    break;
                }
            }
        }
    }

    for (int i = 0 ; i < roots.size() ; ++i)
    {
        answer(nodes[0], roots[i]);
    }

    std::reverse(nodes.begin(), nodes.end()); nodes.pop_back();
    std::sort(nodes.begin(), nodes.end(), [&](int x, int y)
    {
        return dsu.find(x) < dsu.find(y);
    });

    std::vector <int> curr;
    curr.push_back(nodes[0]);
    for (int i = 1 ; i < nodes.size() ; ++i)
    {
        if (dsu.find(nodes[i]) == dsu.find(nodes[i - 1]))
        {
            curr.push_back(nodes[i]);
        } else
        {
            reccc(curr);
            curr.clear();
            curr.push_back(nodes[i]);
        }
    }

    reccc(curr);
}

std::vector <int> perm;
void solve(int N)
{
    n = N;
    perm.resize(n);
    std::iota(perm.begin(), perm.end(), 1);
    reccc(perm);
}
